@@ -44,7 +44,7 @@ use constant {
     IPV6MIB => 3,
 };
 
-$VERSION = '2.06';
+$VERSION = '3.28';
 
 
 
@@ -59,7 +59,7 @@ $VERSION = '2.06';
 %FUNCS = ( 
     'ip_n2p_phys_addr'  => 'ipNetToPhysicalPhysAddress',    # IP-MIB
     'c_inet_phys_addr'  => 'cInetNetToMediaPhysAddress',    # CISCO-IETF-IP-MIB
-    'i6_n2p_phys_addr'  => 'ipv6NetToMediaNetAddress',      # IPV6-MIB
+    'i6_n2p_phys_addr'  => 'ipv6NetToMediaPhysAddress',     # IPV6-MIB
 
     'ip_n2p_phys_type'  => 'ipNetToPhysicalType',           # IP-MIB
     'c_inet_phys_type'  => 'cInetNetToMediaType',           # CISCO-IETF-IP-MIB
@@ -137,8 +137,17 @@ sub ipv6_n2p_addr {
             }
             if ($addrtype == 2) { # IPv6
                 my $v6_packed = pack("C*", split(/\./, $v6addr));
-                $v6addr = join(':', map { sprintf("%04x", $_) } unpack("n*", $v6_packed) );
-                $return->{$row} = $v6addr;
+                if (length($v6_packed) == 15) {
+                    # Workaround for some some IP-MIB implementations, eg on Cisco Nexus: no explicit addrsize, 
+                    # so what we've collected in that variable is actually the first byte of the address.
+                    $v6_packed = pack('C', $addrsize) . $v6_packed;
+                }
+                if (length($v6_packed) == 16) {
+                    $v6addr = join(':', map { sprintf("%04x", $_) } unpack("n*", $v6_packed) );
+                    $return->{$row} = $v6addr;
+                } else {
+                    printf("Invalid size for IPv6 address: expected 16 bytes, got %d (%s = %s)\n", length($v6_packed), $row, $net_addr->{$row});
+                }
             }
         }
     }
@@ -385,8 +394,9 @@ Jeroen van Ingen and Carlos Vicente
 =head1 DESCRIPTION
 
 The SNMP::Info::IPv6 class implements functions to for mapping IPv6 addresses 
-to MAC addresses, interfaces and more. It will use data from the IP-MIB, IPV6-MIB 
-or the CISCO-IETF-IP-MIB, whichever is supported by the device.
+to MAC addresses, interfaces and more. It will use data from the F<IP-MIB>,
+F<IPV6-MIB>, or the F<CISCO-IETF-IP-MIB>, whichever is supported by the
+device.
 
 This class is inherited by Info::Layer3 to provide IPv6 node tracking across  
 device classes.
