@@ -49,7 +49,7 @@ use SNMP::Info::IEEE802dot11;
 
 use vars qw/$VERSION %FUNCS %GLOBALS %MIBS %MUNGE/;
 
-$VERSION = '2.06';
+$VERSION = '3.28';
 
 %GLOBALS = (
     %SNMP::Info::IEEE802dot11::GLOBALS,
@@ -60,7 +60,8 @@ $VERSION = '2.06';
     %SNMP::Info::CiscoConfig::GLOBALS,
     %SNMP::Info::CDP::GLOBALS,
     'serial' => 'entPhysicalSerialNum.1',
-    'descr'  => 'sysDescr'
+    'descr'  => 'sysDescr',
+    'ps1_type' => 'cpoePdCurrentPowerSource'
 );
 
 %FUNCS = (
@@ -97,6 +98,7 @@ $VERSION = '2.06';
     'CISCO-DOT11-ASSOCIATION-MIB'         => 'cDot11ClientSubIfIndex',
     'CISCO-DOT11-SSID-SECURITY-MIB'       => 'cdot11SecVlanNameId',
     'CISCO-VLAN-IFTABLE-RELATIONSHIP-MIB' => 'cviRoutedVlanIfIndex',
+    'CISCO-POE-PD-MIB'                    => 'cpoePdCurrentPowerSource',
 );
 
 %MUNGE = (
@@ -437,6 +439,38 @@ sub i_ssidbcast {
     return $i_ssidbcast;
 }
 
+sub i_ssidmac {
+    my $aironet    = shift;
+    my $partial    = shift;
+    my $mbss_mac_addr = $aironet->mbss_mac_addr();
+
+    # Same logic as i_ssidbcast to return same indexes as i_ssidlist 
+    my $map = {};
+    foreach my $key ( keys %$mbss_mac_addr ) {
+        my ( $interface, @idx ) = split( /\./, $key );
+        $map->{ pack( "C*", @idx ) } = $mbss_mac_addr->{$key};
+    }
+
+    my $i_ssidlist  = $aironet->i_ssidlist();
+    my $i_ssidmac = {};
+    foreach my $key ( keys %$i_ssidlist ) {
+        $i_ssidmac->{$key} = $map->{ $i_ssidlist->{$key} };
+    }
+    return $i_ssidmac;
+}
+
+###
+# PoE status.  The ps1_type is the PoE injector type, which is just
+# a scalar; the status is a little more complex.
+sub ps1_status {
+    my $aironet = shift;
+    my $idx = $aironet->cpoePdCurrentPowerLevel();
+    my $mw = $aironet->cpoePdSupportedPower( $idx );
+    my $descr = $aironet->cpoePdSupportedPowerMode( $idx );
+
+    return sprintf( "%.2fW (%s)", $mw->{$idx} * 0.001, $descr->{$idx} );
+}
+
 1;
 __END__
 
@@ -520,17 +554,13 @@ These are methods that return scalar value from SNMP
 
 =over
 
-=item $aironet->discription()
-
-Adds info from method e_descr() from SNMP::Info::Entity
-
 =item $aironet->vendor()
 
 Returns 'cisco'
 
 =item $aironet->description()
 
-System description
+System description. Adds info from method e_descr() from SNMP::Info::Entity
 
 =back
 
@@ -556,7 +586,7 @@ Returns radio interfaces.
 
 =item $aironet->cd11_mac()
 
-Returns radio interface MAC addresses.
+Returns client radio interface MAC addresses.
 
 =item $aironet->cd11_ssid()
 
@@ -623,6 +653,16 @@ to which they belong.
 With the same keys as i_ssidlist, returns whether the given SSID is
 being broadcast.
 
+=item $aironet->i_ssidmac()
+
+With the same keys as i_ssidlist, returns the Basic service set
+identification (BSSID), MAC address, the AP is using for the SSID. 
+
+=item $aironet ps1_status()
+
+Returns the PoE injector status based on C<cpoePdSupportedPower> and 
+C<cpoePdSupportedPowerMode>.
+
 =back
 
 =head2 Table Methods imported from SNMP::Info::Layer2
