@@ -44,7 +44,7 @@ use SNMP::Info::CDP;
 
 use vars qw/$VERSION %GLOBALS %MIBS %FUNCS %PORTSTAT %MODEL_MAP %MUNGE/;
 
-$VERSION = '2.06';
+$VERSION = '3.28';
 
 %MIBS = (
     %SNMP::Info::Layer3::MIBS,
@@ -376,6 +376,20 @@ sub i_vlan_membership {
     return $i_vlan_membership;
 }
 
+sub i_vlan_membership_untagged {
+    my $hp  = shift;
+    my $partial = shift;
+
+    my $vlans = $hp->i_vlan($partial);
+    my $i_vlan_membership = {};
+    foreach my $port (keys %$vlans) {
+        my $vlan = $vlans->{$port};
+        push( @{ $i_vlan_membership->{$port} }, $vlan );
+    }
+    
+    return $i_vlan_membership;
+}
+
 sub set_i_vlan {
     my $hp = shift;
     my $rv;
@@ -652,7 +666,8 @@ Returns reference to hash of IIDs to admin duplex setting.
 
 =item $hp->vendor_i_type()
 
-Returns reference to hash of IIDs to HP specific port type (hpSwitchPortType).
+Returns reference to hash of IIDs to HP specific port type
+(C<hpSwitchPortType>).
 
 =item $hp->i_name()
 
@@ -679,6 +694,12 @@ It is the union of tagged, untagged, and auto ports.
     print "Port: $port VLAN: $vlan\n";
   }
 
+=item $hp->i_vlan_membership_untagged()
+
+Returns reference to hash of arrays: key = C<ifIndex>, value = array of VLAN
+IDs.  These are the VLANs which are members of the untagged egress list for
+the port.
+
 =item $hp->v_index()
 
 Returns VLAN IDs
@@ -718,4 +739,6 @@ operations.
 
 =item set_i_vlan_tagged()
 
+=back
+
 =cut
