@@ -33,16 +33,18 @@ package SNMP::Info::Layer3::NetSNMP;
 use strict;
 use Exporter;
 use SNMP::Info::Layer3;
+use SNMP::Info::LLDP;
 
-@SNMP::Info::Layer3::NetSNMP::ISA       = qw/SNMP::Info::Layer3 Exporter/;
+@SNMP::Info::Layer3::NetSNMP::ISA       = qw/SNMP::Info::LLDP SNMP::Info::Layer3 Exporter/;
 @SNMP::Info::Layer3::NetSNMP::EXPORT_OK = qw//;
 
 use vars qw/$VERSION %GLOBALS %MIBS %FUNCS %MUNGE/;
 
-$VERSION = '2.06';
+$VERSION = '3.28';
 
 %MIBS = (
     %SNMP::Info::Layer3::MIBS,
+    %SNMP::Info::LLDP::MIBS,
     'UCD-SNMP-MIB'       => 'versionTag',
     'NET-SNMP-TC'        => 'netSnmpAgentOIDs',
     'HOST-RESOURCES-MIB' => 'hrSystem',
@@ -50,13 +52,20 @@ $VERSION = '2.06';
 
 %GLOBALS = (
     %SNMP::Info::Layer3::GLOBALS,
+    %SNMP::Info::LLDP::GLOBALS,
     'netsnmp_vers'   => 'versionTag',
     'hrSystemUptime' => 'hrSystemUptime',
 );
 
-%FUNCS = ( %SNMP::Info::Layer3::FUNCS, );
+%FUNCS = (
+    %SNMP::Info::Layer3::FUNCS,
+    %SNMP::Info::LLDP::FUNCS,
+);
 
-%MUNGE = ( %SNMP::Info::Layer3::MUNGE, );
+%MUNGE = (
+    %SNMP::Info::Layer3::MUNGE,
+    %SNMP::Info::LLDP::MUNGE,
+);
 
 sub vendor {
     return 'Net-SNMP';
@@ -173,6 +182,8 @@ Subclass for Generic Net-SNMP devices
 
 See L<SNMP::Info::Layer3> for its own MIB requirements.
 
+See L<SNMP::Info::LLDP> for its own MIB requirements.
+
 =back
 
 =head1 GLOBALS
@@ -210,6 +221,10 @@ Returns ''.
 
 See documentation in L<SNMP::Info::Layer3> for details.
 
+=head2 Globals imported from SNMP::Info::LLDP
+
+See documentation in L<SNMP::Info::LLDP> for details.
+
 =head1 TABLE ENTRIES
 
 These are methods that return tables of information in the form of a reference
@@ -231,6 +246,10 @@ Ignores loopback
 
 See documentation in L<SNMP::Info::Layer3> for details.
 
+=head2 Table Methods imported from SNMP::Info::LLDP
+
+See documentation in L<SNMP::Info::LLDP> for details.
+
 =head1 NOTES
 
 In order to cause SNMP::Info to classify your device into this class, it
