@@ -1,7 +1,6 @@
 # SNMP::Info::Layer3::Passport
-# $Id$
 #
-# Copyright (c) 2008 Eric Miller
+# Copyright (c) 2012 Eric Miller
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -30,6 +29,7 @@
 
 package SNMP::Info::Layer3::Passport;
 
+use warnings;
 use strict;
 use Exporter;
 use SNMP::Info::SONMP;
@@ -43,7 +43,7 @@ use SNMP::Info::Layer3;
 
 use vars qw/$VERSION %GLOBALS %FUNCS %MIBS %MUNGE/;
 
-$VERSION = '2.06';
+$VERSION = '3.28';
 
 %MIBS = (
     %SNMP::Info::Layer3::MIBS, %SNMP::Info::RapidCity::MIBS,
@@ -80,12 +80,12 @@ sub model {
 
     return $id unless defined $model;
 
-    $model =~ s/^rcA//i;
+    $model =~ s/^rc(A)?//i;
     return $model;
 }
 
 sub vendor {
-    return 'nortel';
+    return 'avaya';
 }
 
 sub os {
@@ -126,9 +126,9 @@ sub i_index {
 
     # Get VLAN Virtual Router Interfaces
     if (!defined $partial
-        or (defined $model
-            and (  ( $partial > 2000 and $model =~ /(86|83|81|16)/ )
-                or ( $partial > 256 and $model =~ /(105|11[05]0|12[05])/ ) )
+        || (defined $model
+            && (  ( $partial > 2000 && $model =~ /^8[8631]|16|VSP/ )
+                || ( $partial > 256 && $model =~ /^1[012][05]0/ ) )
         )
         )
     {
@@ -145,7 +145,7 @@ sub i_index {
         }
     }
 
-    if ( defined $model and $model =~ /(86)/ ) {
+    if ( defined $model and $model =~ /^8[86]/ ) {
 
         my $cpu_index = $passport->rc_cpu_ifindex($partial) || {};
         my $virt_ip = $passport->rc_virt_ip();
@@ -182,9 +182,9 @@ sub interfaces {
     my $vlan_id = {};
 
     if (!defined $partial
-        or (defined $model
-            and (  ( $partial > 2000 and $model =~ /(86|83|81|16)/ )
-                or ( $partial > 256 and $model =~ /(105|11[05]0|12[05])/ ) )
+        || (defined $model
+            && (  ( $partial > 2000 && $model =~ /^8[8631]|16|VSP/ )
+                || ( $partial > 256 && $model =~ /^1[012][05]0/ ) )
         )
         )
     {
@@ -198,24 +198,24 @@ sub interfaces {
         my $index = $i_index->{$iid};
         next unless defined $index;
 
-        if ( ( $index == 1 ) and ( $model =~ /(86)/ ) ) {
+        if ( ( $index == 1 ) and ( $model =~ /^8[86]/ ) ) {
             $if{$index} = 'Cpu.Virtual';
         }
 
-        elsif ( ( $index == 192 ) and ( $model eq '8603' ) ) {
+        elsif ( ( $index == 192 ) and ( $model =~ /^8[86]03/ ) ) {
             $if{$index} = 'Cpu.3';
         }
 
-        elsif ( ( $index == 320 ) and ( $model =~ /(8606|8610|8610co)/ ) ) {
+        elsif ( ( $index == 320 ) and ( $model =~ /^8[86][10][06]/ ) ) {
             $if{$index} = 'Cpu.5';
         }
 
-        elsif ( ( $index == 384 ) and ( $model =~ /(8606|8610|8610co)/ ) ) {
+        elsif ( ( $index == 384 ) and ( $model =~ /^8[86][10][06]/ ) ) {
             $if{$index} = 'Cpu.6';
         }
 
-        elsif (( $index > 2000 and $model =~ /(86|83|81|16)/ )
-            or ( $index > 256 and $model =~ /(105|11[05]0|12[05])/ ) )
+        elsif (( $index > 2000 and $model =~ /^8[8631]|16|VSP/ )
+            or ( $index > 256 and $model =~ /^1[012][05]0/ ) )
         {
 
             my $v_index = $reverse_vlan{$iid};
@@ -255,9 +255,9 @@ sub i_mac {
 
     # Get VLAN Virtual Router Interfaces
     if (!defined $partial
-        or (defined $model
-            and (  ( $partial > 2000 and $model =~ /(86|83|81|16)/ )
-                or ( $partial > 256 and $model =~ /(105|11[05]0|12[05])/ ) )
+        || (defined $model
+            && (  ( $partial > 2000 && $model =~ /^8[8631]|16|VSP/ )
+                || ( $partial > 256 && $model =~ /^1[012][05]0/ ) )
         )
         )
     {
@@ -276,7 +276,7 @@ sub i_mac {
         }
     }
 
-    if ( defined $model and $model =~ /(86)/ ) {
+    if ( defined $model and $model =~ /^8[86]/ ) {
 
         my $cpu_mac = $passport->rc_cpu_mac($partial) || {};
         my $virt_ip = $passport->rc_virt_ip()         || '0.0.0.0';
@@ -329,9 +329,9 @@ sub i_description {
 
     # Get VLAN Virtual Router Interfaces
     if (!defined $partial
-        or (defined $model
-            and (  ( $partial > 2000 and $model =~ /(86|83|81|16)/ )
-                or ( $partial > 256 and $model =~ /(105|11[05]0|12[05])/ ) )
+        || (defined $model
+            && (  ( $partial > 2000 && $model =~ /^8[8631]|16|VSP/ )
+                || ( $partial > 256 && $model =~ /^1[012][05]0/ ) )
         )
         )
     {
@@ -365,9 +365,9 @@ sub i_name {
     my %reverse_vlan;
 
     if (!defined $partial
-        or (defined $model
-            and (  ( $partial > 2000 and $model =~ /(86|83|81|16)/ )
-                or ( $partial > 256 and $model =~ /(105|11[05]0|12[05])/ ) )
+        || (defined $model
+            && (  ( $partial > 2000 && $model =~ /^8[8631]|16|VSP/ )
+                || ( $partial > 256 && $model =~ /^1[012][05]0/ ) )
         )
         )
     {
@@ -379,31 +379,31 @@ sub i_name {
     my %i_name;
     foreach my $iid ( keys %$i_index ) {
 
-        if ( ( $iid == 1 ) and ( $model =~ /(86)/ ) ) {
+        if ( ( $iid == 1 ) and ( $model =~ /^8[86]/ ) ) {
             $i_name{$iid} = 'CPU Virtual Management IP';
         }
 
-        elsif ( ( $iid == 192 ) and ( $model eq '8603' ) ) {
+        elsif ( ( $iid == 192 ) and ( $model =~ /^8[86]03/ ) ) {
             $i_name{$iid} = 'CPU 3 Ethernet Port';
         }
 
-        elsif ( ( $iid == 320 ) and ( $model =~ /(8606|8610|8610co)/ ) ) {
+        elsif ( ( $iid == 320 ) and ( $model =~ /^8[86][10][06]/ ) ) {
             $i_name{$iid} = 'CPU 5 Ethernet Port';
         }
 
-        elsif ( ( $iid == 384 ) and ( $model =~ /(8606|8610|8610co)/ ) ) {
+        elsif ( ( $iid == 384 ) and ( $model =~ /^8[86][10][06]/ ) ) {
             $i_name{$iid} = 'CPU 6 Ethernet Port';
         }
 
         elsif (
-            ( $iid > 2000 and defined $model and $model =~ /(86|83|81|16)/ )
+            ( $iid > 2000 and defined $model and $model =~ /^8[8631]|16|VSP/ )
             or (    $iid > 256
                 and defined $model
-                and $model =~ /(105|11[05]0|12[05])/ )
+                and $model =~ /^1[012][05]0/ )
             )
         {
-            my $vlan_index = $reverse_vlan{$iid};
-            my $vlan_name  = $v_name->{$vlan_index};
+            my $vlan_idx = $reverse_vlan{$iid};
+            my $vlan_name  = $v_name->{$vlan_idx};
             next unless defined $vlan_name;
 
             $i_name{$iid} = $vlan_name;
@@ -433,12 +433,16 @@ sub ip_index {
     foreach my $ip ( keys %$ip_index ) {
         my $iid = $ip_index->{$ip};
         next unless defined $iid;
+        # Skip VSP default CPU addresses
+        next if ($ip =~ /^192\.168\.1\.1/);
+        # Skip default CPU addresses
+        next if ($ip =~ /^192\.168\.168\.16[89]/);
 
         $ip_index{$ip} = $iid;
     }
 
     # Only 8600 has CPU and Virtual Management IP
-    if ( defined $model and $model =~ /(86)/ ) {
+    if ( defined $model and $model =~ /^8[86]/ ) {
 
         my $cpu_ip = $passport->rc_cpu_ip($partial) || {};
         my $virt_ip = $passport->rc_virt_ip($partial);
@@ -447,6 +451,8 @@ sub ip_index {
         foreach my $cid ( keys %$cpu_ip ) {
             my $c_ip = $cpu_ip->{$cid};
             next unless defined $c_ip;
+            # Skip default CPU addresses
+            next if ($c_ip =~ /192\.168\.168\.16[89]/);
 
             $ip_index{$c_ip} = $cid;
         }
@@ -467,6 +473,10 @@ sub ip_netmask {
 
     my %ip_index;
     foreach my $iid ( keys %$ip_mask ) {
+        # Skip VSP default CPU addresses
+        next if ($iid =~ /^192\.168\.1\./);
+        # Skip default CPU addresses
+        next if ($iid =~ /^192\.168\.168\.16[89]/);
         my $mask = $ip_mask->{$iid};
         next unless defined $mask;
 
@@ -474,7 +484,7 @@ sub ip_netmask {
     }
 
     # Only 8600 has CPU and Virtual Management IP
-    if ( defined $model and $model =~ /(86)/ ) {
+    if ( defined $model and $model =~ /^8[86]/ ) {
 
         my $cpu_ip    = $passport->rc_cpu_ip($partial)    || {};
         my $cpu_mask  = $passport->rc_cpu_mask($partial)  || {};
@@ -485,6 +495,8 @@ sub ip_netmask {
         foreach my $iid ( keys %$cpu_mask ) {
             my $c_ip = $cpu_ip->{$iid};
             next unless defined $c_ip;
+            # Skip default CPU addresses
+            next if ($c_ip =~ /192\.168\.168\.16[89]/);
             my $c_mask = $cpu_mask->{$iid};
             next unless defined $c_mask;
 
@@ -510,7 +522,7 @@ sub root_ip {
     my $sonmp_topo_ip   = $passport->sonmp_topo_ip();
 
     # Only 8600 and 1600 have CLIP or Management Virtual IP
-    if ( defined $model and $model =~ /(86|16)/ ) {
+    if ( defined $model and $model =~ /^8[86]|16|VSP/ ) {
 
         # Return CLIP (CircuitLess IP)
         foreach my $iid ( keys %$rc_ip_type ) {
@@ -560,7 +572,10 @@ sub index_factor {
 
     # Older Accelar models use base 16 instead of 64
     $index_factor = 16
-        if ( defined $model and $model =~ /(105|11[05]0|12[05])/ );
+        if ( defined $model and $model =~ /^1[012][05]0/ );
+    # Newer VSP 4K uses 192?
+    $index_factor = 192
+        if ( defined $model and $model =~ /^VSP4/ );
     return $index_factor;
 }
 
@@ -583,9 +598,86 @@ sub bp_index {
     foreach my $iid ( keys %$if_index ) {
         $bp_index{$iid} = $iid;
     }
+
+    # If we have MLT's map them to the designated port
+    my $trunks = $passport->rc_mlt_index;
+    my $dps    = $passport->rc_mlt_dp || {};
+
+    if ( ref {} eq ref $trunks and scalar keys %$trunks ) {
+        foreach my $m ( keys %$trunks ) {
+            my $m_idx = $trunks->{$m};
+            next unless $m_idx;
+            my $i_idx = $dps->{$m} ? $dps->{$m} : $m_idx;
+            $bp_index{$m_idx} = $i_idx;
+        }
+    }
+
     return \%bp_index;
 }
 
+# We have devices which support BRIDGE-MIB, Q-BRIDGE-MIB, and RAPID-CITY
+# exclusively.  Use standards-based first and fall back to RAPID-CITY.
+sub fw_mac {
+    my $passport  = shift;
+    my $partial   = shift;
+
+    my $qb = $passport->SUPER::fw_mac($partial);
+    return $qb if (ref {} eq ref $qb and scalar keys %$qb);
+
+    my $qb_fw_port = $passport->rcBridgeTpFdbPort($partial);
+    my $qb_fw_mac  = {};
+    foreach my $idx ( keys %$qb_fw_port ) {
+        my ( $fdb_id, $mac ) = _rc_fdbtable_index($idx);
+        $qb_fw_mac->{$idx} = $mac;
+    }
+    return $qb_fw_mac;
+}
+
+sub fw_port {
+    my $passport  = shift;
+    my $partial   = shift;
+
+    my $qb = $passport->SUPER::fw_port($partial);
+    return $qb if (ref {} eq ref $qb and scalar keys %$qb);
+
+    return $passport->rcBridgeTpFdbPort($partial);
+}
+
+sub fw_status {
+    my $passport  = shift;
+    my $partial   = shift;
+
+    my $qb = $passport->SUPER::fw_status($partial);
+    return $qb if (ref {} eq ref $qb and scalar keys %$qb);
+    
+    return $passport->rcBridgeTpFdbStatus($partial);
+}
+
+sub qb_fw_vlan {
+    my $passport  = shift;
+    my $partial   = shift;
+
+    my $qb = $passport->SUPER::qb_fw_vlan($partial);
+    return $qb if (ref {} eq ref $qb and scalar keys %$qb);
+
+    my $qb_fw_port = $passport->rcBridgeTpFdbPort($partial);
+    my $qb_fw_vlan = {};
+    foreach my $idx ( keys %$qb_fw_port ) {
+        my ( $fdb_id, $mac ) = _rc_fdbtable_index($idx);
+        $qb_fw_vlan->{$idx} = $fdb_id;
+    }
+    return $qb_fw_vlan;
+}
+
+# break up the rcBridgeTpFdbEntry INDEX into FDB ID and MAC Address.
+sub _rc_fdbtable_index {
+    my $idx    = shift;
+    my @values = split( /\./, $idx );
+    my $fdb_id = shift(@values);
+    return ( $fdb_id, join( ':', map { sprintf "%02x", $_ } @values ) );
+}
+
+
 # Pseudo ENTITY-MIB methods
 
 sub e_index {
@@ -613,7 +705,7 @@ sub e_index {
     }
 
     # Older Accelars use RAPID-CITY::rcCardTable
-    if ( defined $model and $model =~ /(105|11[05]0|12[05])/ ) {
+    if ( defined $model and $model =~ /^1[012][05]0/ ) {
         my $rc_c_t = $passport->rc_c_type() || {};
         foreach my $idx ( keys %$rc_c_t ) {
             next unless $idx;
@@ -683,7 +775,7 @@ sub e_descr {
 
     my $model = $passport->model();
     my $rc_ps = $passport->rc_ps_detail() || {};
-    my $rc_ch = $passport->rcChasType();
+    my $rc_ch = $passport->chassis() || '';
     $rc_ch =~ s/a//;
 
     my %rc_e_descr;
@@ -701,7 +793,7 @@ sub e_descr {
     }
 
     # Older Accelars use RAPID-CITY::rcCardTable
-    if ( defined $model and $model =~ /(105|11[05]0|12[05])/ ) {
+    if ( defined $model and $model =~ /^1[012][05]0/ ) {
         my $rc_c_t = $passport->rc_c_type() || {};
         foreach my $idx ( keys %$rc_c_t ) {
             next unless $idx;
@@ -752,7 +844,7 @@ sub e_type {
 
     my $model = $passport->model();
     my $rc_ps = $passport->rc_ps_type() || {};
-    my $rc_ch = $passport->rcChasType();
+    my $rc_ch = $passport->chassis();
 
     my %rc_e_type;
 
@@ -769,7 +861,7 @@ sub e_type {
     }
 
     # Older Accelars use RAPID-CITY::rcCardTable
-    if ( defined $model and $model =~ /(105|11[05]0|12[05])/ ) {
+    if ( defined $model and $model =~ /^1[012][05]0/ ) {
         my $rc_c_t = $passport->rc_c_type() || {};
         foreach my $idx ( keys %$rc_c_t ) {
             next unless $idx;
@@ -842,7 +934,7 @@ sub e_name {
             $rc_e_name{$iid} = "Card $slot, MDA $mod";
         }
         elsif ( defined $model
-            and $model =~ /(105|11[05]0|12[05])/
+            and $model =~ /^1[012][05]0/
             and $iid   =~ /1$/ )
         {
             $rc_e_name{$iid} = "Card $slot";
@@ -878,7 +970,7 @@ sub e_hwver {
     }
 
     # Older Accelars use RAPID-CITY::rcCardTable
-    if ( defined $model and $model =~ /(105|11[05]0|12[05])/ ) {
+    if ( defined $model and $model =~ /^1[012][05]0/ ) {
         my $rc_c_t = $passport->rc_c_rev() || {};
         foreach my $idx ( keys %$rc_c_t ) {
             next unless $idx;
@@ -927,7 +1019,7 @@ sub e_vendor {
 
     my %rc_e_vendor;
     foreach my $iid ( keys %$rc_e_idx ) {
-        $rc_e_vendor{$iid} = 'nortel';
+        $rc_e_vendor{$iid} = 'avaya';
     }
     return \%rc_e_vendor;
 }
@@ -953,7 +1045,7 @@ sub e_serial {
     }
 
     # Older Accelars use RAPID-CITY::rcCardTable
-    if ( defined $model and $model =~ /(105|11[05]0|12[05])/ ) {
+    if ( defined $model and $model =~ /^1[012][05]0/ ) {
         my $rc_c_t = $passport->rc_c_serial() || {};
         foreach my $idx ( keys %$rc_c_t ) {
             next unless $idx;
@@ -1051,8 +1143,8 @@ __END__
 
 =head1 NAME
 
-SNMP::Info::Layer3::Passport - SNMP Interface to modular Nortel Ethernet Routing
-Switches (formerly Passport / Accelar)
+SNMP::Info::Layer3::Passport - SNMP Interface to modular Avaya
+Ethernet Routing Switch 8000 Series and VSP 9000 Series switches.
 
 =head1 AUTHOR
 
@@ -1075,10 +1167,10 @@ Eric Miller
 
 =head1 DESCRIPTION
 
-Abstraction subclass for modular Nortel Ethernet Routing Switches (formerly
-Passport and Accelar Series Switches).
+Abstraction subclass for modular Avaya Ethernet Routing Switch 8000 Series
+(formerly Nortel/Bay Passport/Accelar) and VSP 9000 Series switches.
 
-These devices have some of the same characteristics as the stackable Nortel 
+These devices have some of the same characteristics as the stackable Avaya 
 Ethernet Switches (Baystack).  For example, extended interface information is 
 gleaned from F<RAPID-CITY>.
 
@@ -1126,7 +1218,7 @@ F<RAPID-CITY-MIB> and then parses out C<rcA>.
 
 =item $passport->vendor()
 
-Returns 'nortel'
+Returns 'avaya'
 
 =item $passport->os()
 
@@ -1240,6 +1332,40 @@ problems with F<BRIDGE-MIB>
 
 =back
 
+=head2 Forwarding Table
+
+These methods utilize, in order; F<Q-BRIDGE-MIB>, F<BRIDGE-MIB>, and
+F<RAPID-CITY> to obtain the forwarding table information.
+
+=over 
+
+=item $passport->fw_mac()
+
+Returns reference to hash of forwarding table MAC Addresses
+
+(C<dot1qTpFdbAddress>), (C<dot1dTpFdbAddress>), (C<rcBridgeTpFdbAddress>)
+
+=item $passport->fw_port()
+
+Returns reference to hash of forwarding table entries port interface
+identifier (iid)
+
+(C<dot1qTpFdbPort>), (C<dot1dTpFdbPort>), (C<rcBridgeTpFdbPort>)
+
+=item $passport->fw_status()
+
+Returns reference to hash of forwarding table entries status
+
+(C<dot1qTpFdbStatus>), (C<dot1dTpFdbStatus>), (C<rcBridgeTpFdbStatus>)
+
+=item $passport->qb_fw_vlan()
+
+Returns reference to hash of forwarding table entries VLAN ID
+
+(C<dot1qFdbId>), (C<rcBridgeTpFdbVlanId>)
+
+=back
+
 =head2 Pseudo F<ENTITY-MIB> information
 
 These devices do not support F<ENTITY-MIB>.  These methods emulate Physical
@@ -1273,7 +1399,7 @@ Returns reference to hash.  Key: IID, Value: Hardware version.
 
 =item $passport->e_vendor()
 
-Returns reference to hash.  Key: IID, Value: nortel.
+Returns reference to hash.  Key: IID, Value: avaya.
 
 =item $passport->e_serial()
 
